//+--------------------------------------------------------------------------------------------+
//|                                                                                            |
//|                                SonicR Chart Panel.mq4                                      | 
//|                                                                                            |
//+--------------------------------------------------------------------------------------------+ 
#property copyright "Copyright @ 2011 traderathome"
#property link      "email: traderathome@msn.com" 

/*---------------------------------------------------------------------------------------------
Overview:

The Sonic Chart Panel Full (White or Black).mq4 indicator incorporates a few changes from the
previous SonicR Chart Panel.mq4 indicator that was released 06/15/2011.  A list of the changes
can be found at the bottom of this overview.
 
This indicator places a colored panel in the upper left corner of the chart containing five 
labels: (1) the pair symbol and time frame, (2) the spread, (3) the range (average for the 
period selected, and current for the session), (4) the time to the next candle, and (5) the 
market bid price.  The "SonicR" logo appears in an extension panel to the right.

The indicator can be used for both forex and non-forex items. In the Forex mode, you can show 
either 4 digit or 5 digits.  If your platform is 4 digit and you select the 5 digit display, 
a "0" is added to displayed numbers. If it is a 5 digit platform and you select to display 4 
digits, numbers are rounded off to 4 digits, except for the Bid display, which just drops the
fifth digit and will not increment/decrement until the next whole 4 digit number is reached. 

This indicator is a multi-function indicator.  It optionally charts: (1) a special bid line,
(2) a background of level lines that equally divide the distance between whole numbers into 
quarters, (3) the Daily Central Pivot line, (4) the Daily Suppport & Resistance lines, 
(5) average Range H/L target lines, and (6) day separators.

1. Optional Bid Ray Line-
You can select to place your own Bid Ray Line on the chart, which you can vary in thickness 
and color.  The Bid Ray Line ends at the current candle, or at your specified number of TFs 
beyond.  It terminates with a Bid Dot that can be sized and that changes color(grn/red) as 
the bid price changes(up/dn).   To display only your own Bid Ray Line, right click on the 
chart, select Chart Properties/Colors and make the Grid color = "None".  Otherwise, the MT4 
bid line is beneath your own, and visible from the end of your line to the chart edge.

2. Optional Level Lines-
This indicator places level lines on your chart at key levels between whole numbers (00, 25, 
50, 75 and 00 again).  The number of "sets" of these lines above and below the current price 
range "set" is determined by hard code.  But "Override_Line_Sets_Displayed" allows for change.  
For example, if "1" is entered, then one set of lines is displayed for the current range.  If 
"2" is entered, then one set of lines is displayed for the current range and a second set is 
displayed both above and below the current range.  An input of "0" or less will default to the 
hard code.  The lines can individually be of any available color and style and thickness. 
The half number levels and quarter number levels can be turned On/Off individually while this 
option is displaying the whole nummber levels.   

3. Optional Daily Pivot Line-
You can select to display the DPV line, which is actually constructed using two lines.  One 
is a wide background line of a highlighting color (Example: Yellow) and the other is a narrow 
top line of a darker color (Example: Black).  You have your choice of line colors, styles and 
thickness (for solid lines).

4. Optional Daily Support & Resistance Lines-
You can select to display the DSR lines, which are extended to include five levels instead of
the usual three levels.  You have your choice of line colors, styles and thickness (for solid
lines).  

5. Optional Range High/Low Lines-
You can select to display two horizontal lines, one for the computed range high target, and 
the other for the computed range low target for the session.  The computed range is based on 
an averaging period you select (default = 15 days).  The range averaging process skips brief 
Sunday sessions, which would drag the average down.  

Two conditions determine where these lines appear....
  1. Condition #1 - the session range has not exceeded the computed average range. 
     * The RgH line is the computed average range distance above the session Low.
     * The RgL line is the computed average range distance below the session high.  
     * The lines will move as new highs/lows are achieved during the session.  
     * This display shows how far price can move in either direction before exceeding the 
       computed average range.  
  2. Condition #2 - the session range has exceeded the computed average range. 
     * RgH is the session high, if it came before the session low, and RgL is the computed
       average range below RgH.
     * RgL is the session low, if it came before the session high, and RgH is the computed
       average range above RgL.
     * The lines lock in these postions for the balance of the session.
     * This display shows the first range breakout, and subsequent price activity. 
      
The purpose of the RgH/RgL lines is to provide a perspective on how far PA might move during
the trading day, based on the average of course.  This can be of some guidance in selecting
where to exit a trade.  A swing trader might not heed these lines, and hold a trade into 
oncoming days.  However, a day trader looking for quick profits and limited exposure to 
market price swings, can benefit by selecting a trade exit within the limits of these lines.
 
6. Optional Separators-
You can apply day separators to your chart with this option that lets you select color and 
line styles. 

Displaying Horizontal Lines-
The horizontal line groups can be displayed differently, one from another, using display
numbers 0-4.  The horizontal line groups are: whole number Levels, half number Levels, 
quarter number Levels, the Daily central pivot, Daily S&R pivots, and Range H/L Targets. 
 
  "0" draws the lines across the chart.  
  "1" draws the lines from chart left thru the current session.  
  "2" draws the lines only thru the current session.  
  "3" draws the lines from the start of the current session to chart right.  
  "4" draws the lines from the current candle to chart right. 
   
This last option allows you to shift the start of the lines further away from the current 
candle using the input "Display_4_Shift_Line_Right".  The feature to start lines further right 
from the current candle begins lines at a specific time in the future based on the number 
(of TFs) you input to "Display_4_Shift_Line_Right".  If you are using a chart with the standard 
chart zoom and you input "46" (TFs) to shift lines to the right, and then you zoom in 1 step, 
the lines may disappear because they start too far into the future to be seen on the wider 
candle chart!  To make the lines start in about the same location relative to the right 
chart border on both charts, a lower number would need to be used for the wider candle chart.  
Notes in the External Inputs provide sample numbers that start lines at an similar distance 
in from the right chart border for the various chart zoom settings.  When a "0" setting is
used for "Display_4_Shift_Line_Right" lines begin at the last candle, and you have a choice
of where to place the labels; either at the beginning of the line, or MaxRight.  To place the
labels at the beginning of the line, make "Line_Labels_MaxRight" = false, and the value for
"MaxRight_Shift_Label_Right" does not matter.  To place the labels MaxRight on the line, make 
"Line_Labels_MaxRight" = true, and input a value for "MaxRight_Shift_Label_Right" the same as
the value used for "Display_4_Shift_Line_Right" when using that mode, which would be "42" for
the normal chart zoom setting.


List of Changes From 06/15/2011 Release to 07/16/2011 Release:
01 - limit Levels to Daily and smaller TFs, and add labels feature for Levels
02 - changed Display_4_Shift_Right numbers to accomdate wider 5 digit display, overlooked before
03 - changed Max_Right label placement coding to accomodate Display_4_Shift_Right coding
04 - revised how condition #2 displays RgH and RgL, computes just once & locks lines
05 - removed code for time zone shifted pivots & vLines, & requirement for GMT/DST inputs


                                                                    - Traderathome, 07-16-2011                                                                    
------------------------------------------------------------------------------------------------
Acknowlegements:

CaveMan - who generously devoted his time and programming skills to code pivots so that 
          the indicator can handle any situation where there are gaps between trading days, 
          where there are partial trading days, or both.  This keeps pivot lines from bunching
          up, which otherwise some broker server pivots will sometimes do at the beginnings of
          each week, and as a result of holidays. 
              
Domas4  - "THANKS!" are owed to Domas4 for providing price labeling code compatible with non-
          forex items. 

------------------------------------------------------------------------------------------------
Suggested Settings:          White Chart              Black Chart 
        
Panel_Background_Color       C'233,233,252'           C'18,18,18'
Symbol_And_TF_Color          Black                    C'208,208,208'
Spread_Color                 Black                    C'150,150,150'
Range_Color                  Black                    C'150,150,150'
Candle_Time_Color            Black                    C'150,150,150'
PriceLabel_UP_Color          ForestGreen              C'46,188,46'
PriceLabel_DOWN_Color        Crimson                  C'234,0,0'
sonicS                       C'44,44,255'             C'0,121,230'
sonicM                       C'70,70,70'              LightGray
sonicR                       Red                      Red
Bid_Ray_Color                Black                    Gray 
Bid_Dot_Up_Color             LimeGreen                Lime
Bid_Dot_Dn_Color             OrangeRed                Red  
Level_Whole_Line_Color       Lavender                 C'41,33,73'       
Level_Half_Line_Color        C'207,250,207'           C'0,53,0'             
Level_Quarter_Line_Color     C'242,238,227'           C'54,40,24'                                                  
DPV1_Background_Color        Yellow                   CLR_NONE 
DPV2_Top_Color               Black                    C'230,216,0' 
Pivot_Resistance_Color       C'255,53,17'             C'179,0,0'   
Pivot_Support_Color          C'64,64,255'             C'90,75,173'         
RgH_Color                    C'255,90,62'             Red
RgL_Color                    C'106,106,255'           C'0,115,230'
Line_Labels_Color            MidnightBlue             C'148,148,148'  
Separators_Color             Silver                   C'79,79,0'                                                                                                                                                       
----------------------------------------------------------------------------------------------*/


//+--------------------------------------------------------------------------------------------+
//| Indicator Global Inputs                                                                    |                                                        
//+--------------------------------------------------------------------------------------------+ 
#property indicator_chart_window

//Global External Inputs------------------------------------------------------------------------ 
extern string Part_1                          = "Indicator Master Controls:";
extern bool   Indicator_On                    = true;
extern bool   Main_Panel_On                   = true;
extern bool   __SonicR_Logo_On                = true;
extern int    Main_Panel_Max_TF               = 43200;
extern bool   Bid_Line_On                     = true;
extern bool   Levels_On                       = true;
extern int    Levels_Max_TF                   = 1440;
extern bool   Pivots_DPV_Line_On              = true;
extern bool   Pivots_DSR_Lines_On             = true;
extern int    Pivots_Max_TF                   = 60;  
extern bool   Range_Lines_On                  = true; 
extern int    Range_Lines_Max_TF              = 240;
extern bool   Separators_On                   = false;
extern int    Separators_Max_TF               = 240;
extern string TF_Choices_1                    = "M1 - H4 =  1, 5, 15, 30, 60, 240";
extern string TF_Choices_2                    = "D  W  M =  1440,  10080,  43200";

extern string __                              = "";
extern string Part_2                          = "Chart Panel Settings:";
extern bool   Use_For_Forex                   = true;
extern bool   Display_5_Digits_Forex          = false;
extern int    Digits_To_Show_In_Spread        = 2;
extern color  Panel_Background_Color          = C'233,233,252';
extern color  Symbol_And_TF_Color             = Black; 
extern color  Spread_Color                    = Black;
extern int    Days_For_Range_Average          = 15;  
extern color  Range_Color                     = Black;
extern color  Candle_Time_Color               = Black;
extern color  PriceLabel_UP_Color             = ForestGreen;
extern color  PriceLabel_DOWN_Color           = Crimson;
extern color  sonicS                          = C'44,44,255';
extern color  sonicM                          = Black;
extern color  sonicR                          = Red;

extern string ___                             = "";
extern string Part_3                          = "Bid Ray Line Settings:";
extern color  Bid_Ray_Color                   = Black;
extern int    Bid_Ray_LineStyle_01234         = 0;  
extern int    Bid_Ray_Thickness               = 1;
extern int    Bid_Ray_TFs_Past_Last_Candle    = 4;
extern color  Bid_Dot_Up_Color                = Lime;
extern color  Bid_Dot_Dn_Color                = OrangeRed;     
extern int    Bid_Dot_Size                    = 10;

extern string ____                            = "";
extern string Part_4                          = "Horizontal Lines Position Settings:";
extern string note0                           = "0 = Full screen";
extern string note1                           = "1 = left thru current session";
extern string note2                           = "2 = Current session only";
extern string note3                           = "3 = Current session to right";
extern string note4                           = "4 = Last candle to right (+Shift_Right)";
extern int    Level_Whole_Display_Number      = 4;
extern int    Level_Half_Display_Number       = 4;
extern int    Level_Quarter_Display_Number    = 4;
extern int    Daily_PV_Line_Display_Number    = 4;
extern int    Daily_RS_Line_Display_Number    = 4;
extern int    Range_Line_Display_Number       = 4;
extern string note5                           = "Sample equivalent '+Shift_Right'";
extern string note6                           = "numbers for Display Number 4:";
extern string note7                           = "Zoom out 2 = 164";
extern string note8                           = "Zoom out 1 = 82";
extern string note9                           = "Std. chart = 42";
extern string note10                          = "Zoom in 1  = 22";
extern string note11                          = "Zoom in 2  = 12";
extern string note12                          = "Zoom in 3  = 6";
extern int    Display_4_Shift_Line_Right      = 42;

extern string _____                           = "";
extern string Part_5                          = "Levels Lines Settings:";
extern int    Override_Line_Sets_Displayed    = 0;
extern bool   Level_Half_Lines_On             = true;
extern bool   Level_Quarter_Lines_On          = true;
extern color  Level_Whole_Line_Color          = Lavender;  
extern int    Line1_Style_01234               = 0;
extern int    Line1_SolidThickness            = 3;
extern color  Level_Half_Line_Color           = C'207,250,207'; 
extern int    Line2_Style_01234               = 0;
extern int    Line2_SolidThickness            = 2;
extern color  Level_Quarter_Line_Color        = C'242,238,227';   
extern int    Line3_Style_01234               = 0;
extern int    Line3_SolidThickness            = 2;

extern string ______                          = "";
extern string Part_6                          = "Pivot Lines Settings:"; 
extern color  DPV1_Background_Color           = Yellow;
extern int    DPV1_LineStyle_01234            = 0;
extern int    DPV1_SolidLineThickness         = 3; 
extern color  DPV2_Top_Color                  = Gray;
extern int    DPV2_LineStyle_01234            = 0;
extern int    DPV2_SolidLineThickness         = 1; 
extern color  Pivot_Resistance_Color          = C'255,53,17'; 
extern int    R_LineStyle_01234               = 0;
extern int    R_SolidLineThickness            = 1; 
extern color  Pivot_Support_Color             = C'64,64,255';
extern int    S_LineStyle_01234               = 0;
extern int    S_SolidLineThickness            = 1; 

extern string _______                         = "";
extern string Part_7                          = "Range Lines Settings:"; 
extern color  RgH_Color                       = C'255,90,62';
extern color  RgL_Color                       = C'106,106,255';
extern int    RG_LineStyle_01234              = 0;    
extern int    RG_SolidLineThickness           = 3;

extern string ________                        = "";
extern string Part_8                          = "Horizontal Lines Labels Settings:";
extern color  Line_Labels_Color               = MidnightBlue;
extern bool   LVL_Labels_Shown                = false;
extern bool   DPV_Label_Shown                 = false;
extern bool   DSR_Labels_Shown                = false;
extern bool   RG_Labels_Shown                 = true;
extern bool   Line_Labels_Prices              = false;
extern string Line_Labels_FontStyle           = "Verdana"; 
extern int    Line_Labels_FontSize            = 8;  
extern bool   Line_Labels_MaxRight            = false;
extern int    MaxRight_Shift_Label_Right      = 42;  

extern string _________                       = "";
extern string Part_9                          = "Separators Selections:";
extern color  Separators_Color                = Silver;  
extern int    Separators_Style_01234          = 2;    
extern int    Separators_Solid_Thickness      = 1;
extern bool   Separators_Thru_SubWindows      = true;
extern int    Prior_Days_To_Show_Separators   = 0;


//Buffers, Constants and Variables-------------------------------------------------------------
bool       FLAG_deinit;

//Deinit Section
int        obj_total,k;
string     name;

//Chart Panel
color      Static_Price_Color, Static_Bid_Color, Bid_Dot_Color, Static_Bid_Dot_Color;
int        fxDigits,modifier,Color,Factor,h,i, m,s,t,T1,T2;
int        p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15;
int        p21,p22,p23,p24,p25,p26,p27,p28,p29,p30,p31,p32,p33;
double     Poin, PoinFx, Old_Price, fxPrice, level, Spread, ARg, Range, TodayRange;  
string     item1  = "[Chart Panel] Box 2";  
string     item2  = "[Chart Panel] Box 3"; 
string     item3  = "[Chart Panel] Box 4";
string     item4  = "[Chart Panel] Box 5";
string     item6  = "[Chart Panel] ID&TF";
string     item7  = "[Chart Panel] Spread"; 
string     item8  = "[Chart Panel] Range";     
string     item9  = "[Chart Panel] Time"; 
string     item10 = "[Chart Panel] Price";
string     item11 = "[Chart Panel] Logo1 S"; 
string     item12 = "[Chart Panel] Logo1 o"; 
string     item13 = "[Chart Panel] Logo1 n";     
string     item14 = "[Chart Panel] Logo1 i"; 
string     item15 = "[Chart Panel] Logo1 c";
string     item16 = "[Chart Panel] Logo1 R"; 
string     item17 = "[Chart Panel] Bid Line";
string     item18 = "[Chart Panel] Bid Dot";
string     sub1,sub2,Price,C,G,tab,tab2,timeleft; 

//Levels
color      linecolor;
double     linelevel, linestyle, linewidth, BL;
int        j, Z;
int        Line1_Color, Line2_Color, Line3_Color;
int        u1=00, u2=50, u3=25, u4=75;        
int        c1, ssp, ssp1, NumberOfRanges;
string     linename;

//Range
int        ii,iii,x,xx,r,Hbarshift,Lbarshift,rc;
double     HiToday,LoToday,startToday,endToday;
double     RangeAvg, RangeHigh, RangeLow, RH, RL;

//Pivots
double     YesterdayBar, YesterdayHigh, YesterdayLow, YesterdayClose, Pivot;
double     r1,r2,r3,r4,r5,s1,s2,s3,s4,s5; 
int        idxfirstbartoday, idxlastbartoday, idxfirstbaryesterday, idxlastbaryesterday;  
double     today_high, today_low, today_open, yesterday_high, yesterday_open, yesterday_low, yesterday_close;   
int        barsperday, dayofweektoday, dayofweektofind;  

//Separators
int        bps;
datetime   T3;
string     vline,vlabel;

//Draw Lines
int        a,b,c,R2,T,T4;  
string     line, dt;
datetime   startline, stopline;

//Draw Labels
string     linelabel, spc;
datetime   startlabel, startrangeday; 

//+-------------------------------------------------------------------------------------------+
//| Indicator Initialization                                                                  |                                                        
//+-------------------------------------------------------------------------------------------+      
int init()
   {
   //If one, or both, Range Line settings are invalid, exit------------------------------------
   if (Level_Whole_Display_Number <0 || Level_Whole_Display_Number >4 ||
      Level_Half_Display_Number <0 || Level_Half_Display_Number >4 ||
      Level_Quarter_Display_Number <0 || Level_Quarter_Display_Number >4 ||
      Daily_PV_Line_Display_Number <0 || Daily_PV_Line_Display_Number >4 ||        
      Daily_RS_Line_Display_Number <0 || Daily_RS_Line_Display_Number >4 ||
      Range_Line_Display_Number <0 || Range_Line_Display_Number >4)         
      {
      Alert("Sonic Chart Panel: Range/SR Line Number invalid"); return(false);
      }   
   
   FLAG_deinit = false;    
                
   //Chart Panel - setting vertical & horizontal positions of borders and text   
   G= "g";
   p1= 69; p2= 138; p3= 141; p4= 143; p5= 63; p6= 157; p7= 49; p8= 168; p9= 38;
   p10= 182; p11= 28; p12= 190; p13= 16; p14= 201; p15= 1; 
   p21= 69; p22= 46; p23= 2; p24= 21; p25= 19; p26= 21; p27= 33; p28= 21; p29= 47;
   p30= 5; p31= 61; p32= 9; p33= 18;
      
        
   //Chart Panel - determine display of Market Price
   if (Use_For_Forex)
      {
      sub1=StringSubstr(Symbol(), 3, 3);
      sub2=StringSubstr(Symbol(), 0, 2);
      if (Display_5_Digits_Forex) 
         {
         fxDigits = 5;
         if (sub1 == "JPY") {fxDigits = 3;} 
         if (sub2 == "XA")  {fxDigits = 2;}
         }        
      else 
         {
         fxDigits = 4;
         if (sub1 == "JPY") {fxDigits = 2;}
         if (sub2 == "XA")  {fxDigits = 2;} 
         }         
      } 
   if (Digits == 5 || Digits == 3) {Factor = 10;} 
   else {Factor = 1;} //cater for 5 digits 
    
   //Chart Panel and Levels - Check for unconventional Point digits number for display of market price
   if (Point == 0.00001) {Poin = 0.0001;} //5 digits     
   else if (Point == 0.001) {Poin = 0.01;} //3 digits
   else {Poin = Point;} //Normal 4 digits
   if (Symbol() == "XAUUSD") {Poin = 0.1;}  
   
   //Levels - set ranges to user specified #, or use a preset number
   if ((Levels_On) && (Override_Line_Sets_Displayed >= 1)) 
      {
      NumberOfRanges = Override_Line_Sets_Displayed-1;
      }   
   else if(Period() == 1) {NumberOfRanges = 4;}
   else if(Period() == 5) {NumberOfRanges = 7;}
   else if(Period() == 15) {NumberOfRanges = 10;}
   else if(Period() == 30) {NumberOfRanges = 10;}   
   else if(Period() == 60) {NumberOfRanges = 10;}  
   else if(Period() == 240) {NumberOfRanges = 20;}  
   else if(Period() == 1440) {NumberOfRanges = 30;} 
   else if(Period() == 10080) {NumberOfRanges = -1;} 
   
   return(0);
   }

//+-------------------------------------------------------------------------------------------+
//| Indicator De-initialization                                                               |                                                        
//+-------------------------------------------------------------------------------------------+       
int deinit()
   {   
   obj_total= ObjectsTotal();  
   for (k= obj_total; k>=0; k--)
      {
      name= ObjectName(k); 
      if (StringSubstr(name,0,13)=="[Chart Panel]"){ObjectDelete(name);}
      }
   //Comment ("");
   return(0);
   }

//+-------------------------------------------------------------------------------------------+
//| Indicator Start                                                                           |                                                        
//+-------------------------------------------------------------------------------------------+         
int start()
   {
   //If Indicator is "Off" deinitialize only once, not every tick------------------------------  
   if (!Indicator_On) 
      {
      if (!FLAG_deinit) {deinit(); FLAG_deinit = true;}
      return(0);
      }
            
   //If indicator is "ON", deinitialize and run------------------------------------------------ 
   deinit(); FLAG_deinit = false; 

   //Panel and Range lines - Define today's bar/data-------------------------------------------                   
	int shift = iBarShift(NULL,PERIOD_D1,Time[0]) + 0;	//today bar= +0, yesterday bar= +1	 
	HiToday   = iHigh(NULL,PERIOD_D1,shift);
	LoToday	 = iLow (NULL,PERIOD_D1,shift);	
	Hbarshift = iHighest(NULL,0,MODE_HIGH,1440/Period(),shift); 
	Lbarshift = iLowest(NULL,0,MODE_LOW,1440/Period(),shift);     
   TodayRange = ((HiToday - LoToday)/Point)/Factor; 
   
   //Up/down color selection when bid price changes-------------------------------------------- 
   //The color is to remain as-is until price changes direction by a whole pip, or by a fractional
   //pip if 5 digits is displayed.  We don't want a 3rd color on ticks without a price change.
   if (Use_For_Forex)
      {  
      if (Display_5_Digits_Forex) {fxPrice = Bid;}        
      else 
         {
         PoinFx = Poin;         
         if (sub2 == "XA") {PoinFx = 0.01;}
         fxPrice = MathFloor(Bid/PoinFx)*PoinFx;
         }          
      Price=DoubleToStr(fxPrice, fxDigits);
      }      
   else 
      {
      fxPrice = Bid;
      Price=DoubleToStr(Bid, Digits);
      } 
                 
   if (fxPrice > Old_Price) 
      {
      Color = PriceLabel_UP_Color;
      Static_Price_Color = Color;     
      Bid_Dot_Color = Bid_Dot_Up_Color;       
      Static_Bid_Dot_Color = Bid_Dot_Up_Color;   
      }
   else if (fxPrice < Old_Price) 
      {      
      Color = PriceLabel_DOWN_Color; 
      Static_Price_Color = Color;  
      Bid_Dot_Color = Bid_Dot_Dn_Color;  
      Static_Bid_Dot_Color = Bid_Dot_Dn_Color;  
      }   
   else //if (fxPrice == Old_Price)  
      {
      Color = Static_Price_Color;       
      Bid_Dot_Color = Static_Bid_Dot_Color;
      }
   Old_Price = fxPrice;
      
   //Main Panel--------------------------------------------------------------------------------
   if ((Main_Panel_On) && (Period()<= Main_Panel_Max_TF)) {      
   //Create Chart Main Panel Background boxes      
   ObjectCreate(item1, OBJ_LABEL, 0, 0, 0, 0, 0);
   ObjectSetText(item1, G, p21, "Webdings");
   ObjectSet(item1, OBJPROP_CORNER, 0);
   ObjectSet(item1, OBJPROP_XDISTANCE, 0);
   ObjectSet(item1, OBJPROP_YDISTANCE, 0);   
   ObjectSet(item1, OBJPROP_COLOR, Panel_Background_Color);
   ObjectSet(item1, OBJPROP_BACK, false);     
   //-----         
   ObjectCreate(item2, OBJ_LABEL, 0, 0, 0, 0, 0);
   ObjectSetText(item2, "g", p21, "Webdings");
   ObjectSet(item2, OBJPROP_CORNER, 0);
   ObjectSet(item2, OBJPROP_XDISTANCE, p22);//52
   ObjectSet(item2, OBJPROP_YDISTANCE, 0);   
   ObjectSet(item2, OBJPROP_COLOR, Panel_Background_Color);
   ObjectSet(item2, OBJPROP_BACK, false);  
         
   //Symbol and Time Frame   	 	         
   C= " "; if (Period()== 1)  C =C +  "     "+Symbol()+"   M1";
      else {if (Period()== 5)  C =C +  "     "+Symbol()+"   M5";
      else {if (Period()== 15) C =C +  "    "+Symbol()+"   M15";
      else {if (Period()== 30)  C =C +  "    "+Symbol()+"   M30";
      else {if (Period()== 60)  C =C +  "     "+Symbol()+"   H1";
      else {if (Period()== 240)  C =C +  "     "+Symbol()+"   H4";
      else {if (Period()== 1440)  C =C +  "   "+Symbol()+"   Daily";
      else {if (Period()== 10080) C =C +  " "+Symbol()+"   Weekly";
      else {if (Period()== 43200) C =C +  ""+Symbol()+"   Monthly"; }}}}}}}}                          
   ObjectCreate(item6, OBJ_LABEL, 0, 0, 0);
   ObjectSet(item6,OBJPROP_CORNER, 0);
   ObjectSet(item6,OBJPROP_XDISTANCE, 0);
   ObjectSet(item6,OBJPROP_YDISTANCE, p23);
   ObjectSet(item6, OBJPROP_COLOR, Symbol_And_TF_Color);         
   ObjectSetText(item6, C, 11, "Arial Bold");
                         
   //Spread
   Spread = MarketInfo(Symbol(), MODE_SPREAD);
   Spread = Spread/Factor;                                        
   ObjectCreate(item7, OBJ_LABEL, 0, 0, 0);
   ObjectSet(item7,OBJPROP_CORNER, 0);
   ObjectSet(item7,OBJPROP_XDISTANCE, p24);
   ObjectSet(item7,OBJPROP_YDISTANCE, p25);
   ObjectSet(item7, OBJPROP_COLOR, Spread_Color);         
   ObjectSetText(item7, "Spread   " + DoubleToStr(Spread,Digits_To_Show_In_Spread), p32, "Arial"); 
   
   //Range  
   Ranges(Days_For_Range_Average);
   Range = ((ARg/Days_For_Range_Average)/Point)/Factor; 
   ObjectCreate(item8, OBJ_LABEL, 0, 0, 0);
   ObjectSet(item8,OBJPROP_CORNER, 0);
   ObjectSet(item8,OBJPROP_XDISTANCE, p26);
   ObjectSet(item8,OBJPROP_YDISTANCE, p27);
   ObjectSet(item8, OBJPROP_COLOR, Range_Color);              
   ObjectSetText(item8, "Range    " +DoubleToStr(Range,0)+", "+DoubleToStr(TodayRange,0), 9, "Arial");           
                           
   //Time To Next Candle	   
   if (Period()>1440) {timeleft = " (OFF)";}
   else {timeleft =TimeToStr(Time[0]+Period()*60-TimeCurrent(),TIME_MINUTES|TIME_SECONDS);}                 
   ObjectCreate(item9, OBJ_LABEL, 0, 0, 0);
   ObjectSet(item9,OBJPROP_CORNER, 0);
   ObjectSet(item9,OBJPROP_XDISTANCE, p28);
   ObjectSet(item9,OBJPROP_YDISTANCE, p29);         
   ObjectSet(item9, OBJPROP_COLOR, Candle_Time_Color);        
   ObjectSetText(item9, "Candle   "+timeleft, p32, "Arial");          
      
   //Market Price  
   //Add leading spaces to center price labels
   tab="  "; tab2=" "; if ((Display_5_Digits_Forex) && (sub2 != "XA")) {tab=" "; tab2="";}                  
	     if(Symbol()== "USDMXN"){tab= tab2+"";}	   
   else if(Symbol()== "XAUUSD"){tab= tab2+"";} 	 	     	 
   else if(Symbol()== "USDJPY"){tab= tab2+"  ";}
	else if(Symbol()== "CHFJPY"){tab= tab2+"  ";}
	else if(Symbol()== "AUDJPY"){tab= tab2+"  ";}
	else if(Symbol()== "CADJPY"){tab= tab2+"  ";}
	else if(Symbol()== "XAGUSD"){tab= tab2+"  ";} 
   //Create/move label
   ObjectCreate(item10, OBJ_LABEL, 0, 0, 0);
   ObjectSet(item10,OBJPROP_CORNER, 0);
   ObjectSet(item10,OBJPROP_XDISTANCE, p30); 
   ObjectSet(item10,OBJPROP_YDISTANCE, p31); 
   ObjectSet(item10, OBJPROP_COLOR, Color);     
   ObjectSetText(item10, tab+Price, p33, "Verdana Bold"); 
   }//End Main Panel routine
     
   //Create SonicR Panel
   if((Main_Panel_On) && (__SonicR_Logo_On) && (Period()<= Main_Panel_Max_TF)) {
   //Create Sonic Panel Background boxes
   ObjectCreate(item3, OBJ_LABEL, 0, 0, 0, 0, 0);        
   ObjectSetText(item3,  "g", p1, "Webdings");      
   ObjectSet(item3, OBJPROP_CORNER, 0);
   ObjectSet(item3, OBJPROP_XDISTANCE, p2);
   ObjectSet(item3, OBJPROP_YDISTANCE, 0 );    
   ObjectSet(item3, OBJPROP_COLOR, Panel_Background_Color);         
   ObjectSet(item3, OBJPROP_BACK, false);
   //-----  
   ObjectCreate(item4, OBJ_LABEL, 0, 0, 0, 0, 0);        
   ObjectSetText(item4,  "g", p1, "Webdings");      
   ObjectSet(item4, OBJPROP_CORNER, 0);
   ObjectSet(item4, OBJPROP_XDISTANCE, p3);
   ObjectSet(item4, OBJPROP_YDISTANCE, 0 );    
   ObjectSet(item4, OBJPROP_COLOR, Panel_Background_Color);         
   ObjectSet(item4, OBJPROP_BACK, false); 
   
   //Create logo         
   ObjectDelete(item11);
   ObjectCreate(item11, OBJ_LABEL, 0, 0, 0);
   ObjectSet(item11,OBJPROP_CORNER, 0);
   ObjectSet(item11,OBJPROP_XDISTANCE, p4); 
   ObjectSet(item11,OBJPROP_YDISTANCE, p5);
   ObjectSet(item11, OBJPROP_COLOR, sonicS);  
   ObjectSetText(item11, "S", 16, "Verdana Bold");       
   //-----
   ObjectDelete(item12);
   ObjectCreate(item12, OBJ_LABEL, 0, 0, 0);
   ObjectSet(item12,OBJPROP_CORNER, 0);
   ObjectSet(item12,OBJPROP_XDISTANCE, p6); 
   ObjectSet(item12,OBJPROP_YDISTANCE, p7);
   ObjectSet(item12, OBJPROP_COLOR, sonicM);     
   ObjectSetText(item12, "o", 13, "Verdana Bold");    
   //-----
   ObjectDelete(item13);
   ObjectCreate(item13, OBJ_LABEL, 0, 0, 0);
   ObjectSet(item13,OBJPROP_CORNER, 0);
   ObjectSet(item13,OBJPROP_XDISTANCE, p8); 
   ObjectSet(item13,OBJPROP_YDISTANCE, p9);
   ObjectSet(item13, OBJPROP_COLOR, sonicM);     
   ObjectSetText(item13, "n", 13, "Verdana Bold");  
   //-----
   ObjectDelete(item14);
   ObjectCreate(item14, OBJ_LABEL, 0, 0, 0);
   ObjectSet(item14,OBJPROP_CORNER, 0);
   ObjectSet(item14,OBJPROP_XDISTANCE, p10); 
   ObjectSet(item14,OBJPROP_YDISTANCE, p11);
   ObjectSet(item14, OBJPROP_COLOR, sonicM);     
   ObjectSetText(item14, "i", 13, "Verdana Bold");       
   //-----
   ObjectDelete(item15);   
   ObjectCreate(item15, OBJ_LABEL, 0, 0, 0);
   ObjectSet(item15,OBJPROP_CORNER, 0);
   ObjectSet(item15,OBJPROP_XDISTANCE, p12); 
   ObjectSet(item15,OBJPROP_YDISTANCE, p13);
   ObjectSet(item15, OBJPROP_COLOR, sonicM);     
   ObjectSetText(item15, "c", 13, "Verdana Bold");      
   //-----
   ObjectDelete(item16);   
   ObjectCreate(item16, OBJ_LABEL, 0, 0, 0);
   ObjectSet(item16,OBJPROP_CORNER, 0);
   ObjectSet(item16,OBJPROP_XDISTANCE, p14); //p14=210
   ObjectSet(item16,OBJPROP_YDISTANCE, p15);
   ObjectSet(item16, OBJPROP_COLOR, sonicR);     
   ObjectSetText(item16, "R", 17, "Verdana Bold");
   }//End SonicR Panel routine 
     
   //Bid Line---------------------------------------------------------------------------------- 
   if (Bid_Line_On)
      {
      //Bid line
      T1=Time[0]+(Period()*60*Bid_Ray_TFs_Past_Last_Candle); 
      T2=iTime(NULL, NULL, (Period()*60)); 
                         
      if (ObjectFind(item17) != 0){
         ObjectCreate(item17, OBJ_TREND, 0, T1, Bid, T2, Bid);     
         ObjectSet(item17, OBJPROP_STYLE, Bid_Ray_LineStyle_01234 );
         ObjectSet(item17, OBJPROP_COLOR, Bid_Ray_Color);
         ObjectSet(item17, OBJPROP_WIDTH, Bid_Ray_Thickness);}  
      else{ 
         ObjectMove(item17, 0, T1, level);
         ObjectMove(item17, 1, T2, level);} 
       
      //Bid dot                   
      T1=Time[0]+(Period()*60*(Bid_Ray_TFs_Past_Last_Candle));   
      if (ObjectFind(item18) != 0){
         ObjectCreate(item18, OBJ_TREND, 0, T1, Bid, T1, Bid);     
         ObjectSet(item18, OBJPROP_STYLE, 0);
         ObjectSet(item18, OBJPROP_COLOR, Bid_Dot_Color);
         ObjectSet(item18, OBJPROP_WIDTH, Bid_Dot_Size);} 
      else{ 
         ObjectMove(item18, 0, T1, level);
         ObjectMove(item18, 1, T1, level);}                 
      }//End Bid Line

   //Levels------------------------------------------------------------------------------------
   if (Levels_On && Period()<= Levels_Max_TF)
      {      
      //initializing line colors
      Line1_Color = Level_Whole_Line_Color; 
      Line2_Color = Level_Half_Line_Color;
      Line3_Color = Level_Quarter_Line_Color;
       
      //define needed variables
      ssp1= Bid / Poin;  //bid w/o the decimal
      c1=ssp1%100; //last two significant digits   
      BL=c1; //last two significant digits of starting/reference bid line  
       
      //calculate line levels bottom to top
      for (j= -(100-(100-BL))-(100*NumberOfRanges); j<= (100-BL)+(100*NumberOfRanges); j++)       
         {
         ssp=ssp1+j; 
         c1=ssp%100;        
         if (c1==u1)
            {           
            linestyle = Line1_Style_01234;
            linecolor = Line1_Color;
            linewidth = Line1_SolidThickness;
            linelevel = ssp*Poin;
            DrawLines("LVL", Level_Whole_Display_Number, linelevel, linecolor, linestyle, linewidth);
            }
         if ((Level_Half_Lines_On) && (c1==u2))
            {              
            linestyle = Line2_Style_01234;
            linecolor = Line2_Color;
            linewidth = Line2_SolidThickness;
            linelevel = ssp*Poin;
            DrawLines("LVL", Level_Half_Display_Number, linelevel, linecolor, linestyle, linewidth);
            } 
         if ((Level_Quarter_Lines_On) && ((c1==u3) || (c1==u4)))
            {           
            linestyle = Line3_Style_01234;
            linecolor = Line3_Color;
            linewidth = Line3_SolidThickness;
            linelevel = ssp*Poin;
            DrawLines("LVL", Level_Quarter_Display_Number, linelevel, linecolor, linestyle, linewidth);
            }
         }
      }//End Levels         
                    
   //Daily Pivot Lines------------------------------------------------------------------------- 
   if ((Pivots_DPV_Line_On) || (Pivots_DSR_Lines_On) && (Period()<= Pivots_Max_TF))          
      { 
      PivotDay(idxfirstbartoday, idxfirstbaryesterday,idxlastbaryesterday);   
      
      //Walk forward through yestday's start and collect high/lows within the same day
      yesterday_high= -99999;  // not high enough to remain alltime high
      yesterday_low=  +99999;  // not low enough to remain alltime low 
      for (int idxbar= idxfirstbaryesterday; idxbar>=idxlastbaryesterday; idxbar--)
         {
         // grab first value for open
         if (yesterday_open==0) yesterday_open= iOpen(NULL, PERIOD_H1, idxbar);      
         yesterday_high= MathMax(iHigh(NULL, PERIOD_H1, idxbar), yesterday_high);
         yesterday_low= MathMin(iLow(NULL, PERIOD_H1, idxbar), yesterday_low);      
         yesterday_close= iClose(NULL, PERIOD_H1, idxbar);
         }           
      Pivot = (yesterday_high + yesterday_low + yesterday_close) /3; 
      
      if (Pivots_DSR_Lines_On)
         {       
         r5 = (4*Pivot)+(yesterday_high-(4*yesterday_low));
         r4 = (3*Pivot)+(yesterday_high-(3*yesterday_low));
         r3 = (2*Pivot)+(yesterday_high-(2*yesterday_low));
         r2 = Pivot+(yesterday_high - yesterday_low); 
         r1 = (2*Pivot)-yesterday_low;  
         s1 = (2*Pivot)-yesterday_high;
         s2 = Pivot-(yesterday_high - yesterday_low);    
         s3 = (2*Pivot)-((2* yesterday_high)-yesterday_low);
         s4 = (3*Pivot)-((3* yesterday_high)-yesterday_low);
         s5 = (4*Pivot)-((4* yesterday_high)-yesterday_low);
         }             
      //Daily Central Pivot Line 
      if (Pivots_DPV_Line_On)
         {          
         //Pivot Line data to subroutine                                                              
         DrawLines("Pivot DPV1", Daily_PV_Line_Display_Number, Pivot, DPV1_Background_Color, DPV1_LineStyle_01234, DPV1_SolidLineThickness);
         DrawLines("Pivot DPV2", Daily_PV_Line_Display_Number, Pivot, DPV2_Top_Color, DPV2_LineStyle_01234, DPV2_SolidLineThickness);
         }
      //Daily Support & Resistance Lines   
      if (Pivots_DSR_Lines_On)
         {
         DrawLines("Pivot DR5", Daily_RS_Line_Display_Number, r5, Pivot_Resistance_Color, R_LineStyle_01234, R_SolidLineThickness);  
         DrawLines("Pivot DR4", Daily_RS_Line_Display_Number, r4, Pivot_Resistance_Color, R_LineStyle_01234, R_SolidLineThickness);   
         DrawLines("Pivot DR3", Daily_RS_Line_Display_Number, r3, Pivot_Resistance_Color, R_LineStyle_01234, R_SolidLineThickness);   
         DrawLines("Pivot DR2", Daily_RS_Line_Display_Number, r2, Pivot_Resistance_Color, R_LineStyle_01234, R_SolidLineThickness); 
         DrawLines("Pivot DR1", Daily_RS_Line_Display_Number, r1, Pivot_Resistance_Color, R_LineStyle_01234, R_SolidLineThickness);   
         DrawLines("Pivot DS1", Daily_RS_Line_Display_Number, s1, Pivot_Support_Color, S_LineStyle_01234, S_SolidLineThickness);
         DrawLines("Pivot DS2", Daily_RS_Line_Display_Number, s2, Pivot_Support_Color, S_LineStyle_01234, S_SolidLineThickness);
         DrawLines("Pivot DS3", Daily_RS_Line_Display_Number, s3, Pivot_Support_Color, S_LineStyle_01234, S_SolidLineThickness);  
         DrawLines("Pivot DS4", Daily_RS_Line_Display_Number, s4, Pivot_Support_Color, S_LineStyle_01234, S_SolidLineThickness);
         DrawLines("Pivot DS5", Daily_RS_Line_Display_Number, s5, Pivot_Support_Color, S_LineStyle_01234, S_SolidLineThickness);        
         }          
      }//End Daily Pivots
         
   //Range High/Low lines----------------------------------------------------------------------
   if (Range_Lines_On && Period()<= Range_Lines_Max_TF)
      {         
      Ranges(Days_For_Range_Average);
      RangeAvg = NormalizeDouble(ARg/Days_For_Range_Average,4);                
      RangeHigh = LoToday + RangeAvg;  
      RangeLow  = HiToday - RangeAvg; 

      if (HiToday - LoToday > RangeAvg)
         {
         if (rc == 1) //Pass over first time, but do thereafter
            { 
            RangeHigh = RH;
            RangeLow = RL;        
            } 
         if (rc == 0) //Do this the first and only time
            {            
            if (Time[Lbarshift] < Time[Hbarshift]) {RangeLow  = LoToday; RangeHigh = LoToday + RangeAvg;}
            else {RangeHigh = HiToday; RangeLow = HiToday - RangeAvg;}              
            RH = RangeHigh;
            RL = RangeLow;
            rc = 1;               
            }           
         }
                                
      //Range Lines data to subroutine                                                             
      DrawLines("RgH", Range_Line_Display_Number, RangeHigh, RgH_Color, RG_LineStyle_01234, RG_SolidLineThickness);   
      DrawLines("RgL", Range_Line_Display_Number, RangeLow,  RgL_Color, RG_LineStyle_01234, RG_SolidLineThickness);
      }//End Ranges
     
   //Separators-------------------------------------------------------------------------------   
   if (Separators_On && Period() <= Separators_Max_TF) 
      {
      //Separators for Today and Tomorrow           
      CurrentSeparators("Today", Separators_Color, Separators_Style_01234, Separators_Solid_Thickness);          
      CurrentSeparators("Tomorrow", Separators_Color, Separators_Style_01234, Separators_Solid_Thickness);  
              
      //Separators for prior sessions 
      if (Prior_Days_To_Show_Separators > 0)
         {     
         //Calculate bars per session        
              if (Period()==1) {bps = 1440;}   
         else if (Period()==5) {bps = 288;}  
         else if (Period()==15){bps = 96;}    
         else if (Period()==30){bps = 48;}
         else if (Period()==60){bps = 24;}         
         //Define bar starting prior sessions display      
         shift = iBarShift(NULL,NULL,iTime(NULL,PERIOD_D1,1));                       
         //Execute loop for bars per session X number of prior sessions to show   
         for (i= shift; i<=(shift+(bps*(Prior_Days_To_Show_Separators-1))); i++)       
            {  
            h=TimeHour(Time[i]); 
            m=TimeMinute(Time[i]);   
            if (h==0 && m==0)         
               {                     
               PriorSeparators(i, Separators_Color, 
               Separators_Style_01234, Separators_Solid_Thickness);                             
               }         
            }
         }
      }//End Separators   


   //End of program computations---------------------------------------------------------------        
   return(0);
   }

//+-------------------------------------------------------------------------------------------+
//| Indicator Subroutine To Compute Average Ranges                                            |                                                 
//+-------------------------------------------------------------------------------------------+ 
void Ranges (int period)
   {
   int ii, iii, x, xx;
   //Add ranges over period.  Count number of Sundays and exclude Sunday ranges.         
   ARg = 0; for(i=1; i<=Days_For_Range_Average; i++)
       {
       if (TimeDayOfWeek(iTime(NULL,PERIOD_D1,i))!=0) {
       ARg = ARg + iHigh(NULL,PERIOD_D1,i)- iLow(NULL,PERIOD_D1,i);}
       else {x=x+1;}
       }
   //For number of Sundays, add additional days of range
   for(ii=i+1; ii<i+x+1; ii++) 
       {
       if (TimeDayOfWeek(iTime(NULL,PERIOD_D1,ii))!=0) {       
       ARg = ARg + iHigh(NULL,PERIOD_D1,ii)- iLow(NULL,PERIOD_D1,ii);}
       else {xx=xx+1;}       
       }      
   //If a Sunday reduced added days above, add additional day of range
   for(iii=ii+1; iii<ii+xx+1; iii++) 
       {
       ARg = ARg + iHigh(NULL,PERIOD_D1,iii)- iLow(NULL,PERIOD_D1,iii);
       }                     
   //Return total of ranges for period (Sundays excluded/additional days subsituted)
   return (ARg);   
   }
  
//+-------------------------------------------------------------------------------------------+
//| Subroutine To name and draw Daily Pivot and Range HL lines and line labels                |                                                                                  |
//+-------------------------------------------------------------------------------------------+
void DrawLines(string text, int Line_Selection_Number, double level, color Color, int linestyle, int thickness)
   {
   //Lines=====================================================================================  
   //Name lines 
   text = StringTrimLeft(text);       
   line = "[Chart Panel]  " + text + " Line " + level;  

   //Define variables  
   a = linestyle; b = thickness; c =1; if (a==0)c=b;  
   
   //Where to Start lines
   if ((Line_Selection_Number == 0) || (Line_Selection_Number == 1)) {startline = WindowFirstVisibleBar();} //start chart left
   else if(Line_Selection_Number == 2 || Line_Selection_Number == 3) {startline = iTime(NULL, PERIOD_D1, 0);} //start at day separator                  
   else if(Line_Selection_Number == 4) {T4 = Time[0]+ (Period()*60*Display_4_Shift_Line_Right); startline  = T4;} //start right of last candle
                                                                   
   //Where to Stop Lines   
   if(Line_Selection_Number == 0 || Line_Selection_Number == 3) {stopline = Time[0]; R2 = true;}
   else if(Line_Selection_Number == 4) {stopline = T4 + 86400; R2 = true;}   
   else if ((Line_Selection_Number == 1) || (Line_Selection_Number == 2)) {stopline = iTime(NULL,PERIOD_D1,0)+86400; R2 = false;}
                       
   //Draw lines 
   if (ObjectFind(line) != 0)
      {
      ObjectCreate(line, OBJ_TREND, 0, startline, level, stopline, level);    
      ObjectSet(line, OBJPROP_STYLE, a);
      ObjectSet(line, OBJPROP_COLOR, Color);
      ObjectSet(line, OBJPROP_WIDTH, c);
      ObjectSet(line, OBJPROP_BACK, true); 
      ObjectSet(line, OBJPROP_RAY, R2);
      }
   else
      {
      ObjectMove(line, 0, startline, level);
      ObjectMove(line, 1, stopline, level);
      }
   
   //Labels===================================================================================
   //Exit if label not to be shown 
   if ((!LVL_Labels_Shown) && (StringSubstr(text, 0, 3) == "LVL")) {return (-1);}         
   else if ((!DPV_Label_Shown) && (StringSubstr(text, 6, 3) == "DPV")) {return (-1);}
   else if ((!DSR_Labels_Shown) && ((StringSubstr(text, 6, 2) == "DR") || (StringSubstr(text, 6, 2) == "DS"))) {return (-1);} 
   else if ((!RG_Labels_Shown) && (StringSubstr(text, 0, 2) == "Rg")) {return (-1);} 
   else 
      {      
      //Name label 
      linelabel = "[Chart Panel]  " + StringTrimLeft(text) + " Label" + level; 
          
      //Remove "Pivot" from displayed pivots labels     
      if (StringSubstr(text, 0, 6) == "Pivot ") 
         {
         text = StringSubstr(text, 6, 3);          
         }
           
      //Align labels depending on if showing prices and on digits in price      
      if (Line_Labels_Prices && StrToInteger(text)==0)       
         {
         if (Use_For_Forex) 
            {
            Price=DoubleToStr(level, fxDigits);
            if (Display_5_Digits_Forex) {text = "              " + text + "  " + Price;}  //14
            else {text = "            " + text + "  " + Price;}  //12            
            }         
         else 
            {
            Price=DoubleToStr(level, Digits);
            spc = "         "; for (k= 1; k<= Digits; k++) {spc = spc + " ";}
            text = spc + text + "  " + Price;
            }
         }   
      else {text = " " + text + " ";}      
 
      //Where to put labels
      if ((Line_Selection_Number == 0) || (Line_Selection_Number == 1)) //start mid chart                                
         {
         spc = ""; //00 
         startlabel = Time[WindowFirstVisibleBar()/2];
         }             
      else if (Line_Selection_Number == 2 || Line_Selection_Number == 3)
         {            
         if (Time[WindowFirstVisibleBar()] < iTime(NULL, PERIOD_D1, 0)) //start at day separator
            {
            spc="       "; //07                           
            startlabel= iTime(NULL, PERIOD_D1, 0);  
            }          
         else //start start mid chart     
            {
            spc = ""; //00  
            startlabel = Time[WindowFirstVisibleBar()/2];                         
            } 
         }                       
      else if (Line_Selection_Number == 4) //start at current candle   
         {
        
         if (Display_4_Shift_Line_Right > 0)
            {
            spc="         "; //09                   
            startlabel = T4;          
            }         
         else if (Line_Labels_MaxRight)         
            {
            spc="         "; //09                   
            startlabel = Time[0]+ (Period()*60*MaxRight_Shift_Label_Right);             
            } 
         else
            {
            spc="         "; //09                   
            startlabel = Time[0];   
            }       
         }
                                     
      //Draw labels                            
      if (ObjectFind(linelabel) != 0)
         {
         ObjectCreate(linelabel, OBJ_TEXT, 0, startlabel, level);     
         ObjectSetText(linelabel, spc + text, Line_Labels_FontSize, Line_Labels_FontStyle, Line_Labels_Color);
         ObjectSet(linelabel, OBJPROP_BACK, false);
         }        
      else {ObjectMove(linelabel, 0, startlabel, level);}       
      }
   }
   
//+-------------------------------------------------------------------------------------------+
//| Subroutine to Create Today and Tomorrow Separators                                        |          
//+-------------------------------------------------------------------------------------------+   
void CurrentSeparators(string text, color Color, int linestyle, int thickness) 
   {
   a = linestyle;
   b = thickness;
   c =1; if (a==0)c=b; 
      
   //Define datetime for start of current session vline separator
   if(text == "Today")       
      { 
      dt= TimeYear(iTime(NULL,0,0))+ "." +TimeMonth(iTime(NULL,0,0))+ "." + TimeDay(iTime(NULL,PERIOD_D1,0))+ "." + 0 + ":" + "0";                                        
      T3 = StrToTime(dt)+60;            
      vline  = "[Chart Panel] Today Session Line";     
      } 
      
   //Define datetime for end of current session vline separator
   else                
      {
      dt= TimeYear(iTime(NULL,0,0))+ "." +TimeMonth(iTime(NULL,0,0))+ "." + TimeDay(iTime(NULL,PERIOD_D1,0))+ "." + 24 + ":" + "60";                                        
      T3 = StrToTime(dt)+60;              
      vline  = "[Chart Panel] Tomorrow Session Line";            
      }
                        
   if (ObjectFind(vline) != 0){
      if(!Separators_Thru_SubWindows) {ObjectCreate(vline, OBJ_TREND, 0, T3, 0, T3, 100);}  
      else {ObjectCreate(vline, OBJ_VLINE, 0, T3, 0);}
      ObjectSet(vline, OBJPROP_STYLE, a);    
      ObjectSet(vline, OBJPROP_WIDTH, c);
      ObjectSet(vline, OBJPROP_COLOR, Color);
      ObjectSet(vline, OBJPROP_BACK, true); }
   else{
      ObjectMove(vline, 0, T3, 0);} 
   }
      
//+-------------------------------------------------------------------------------------------+
//| Subroutine to Create Prior Sessions Separators                                            |          
//+-------------------------------------------------------------------------------------------+   
void PriorSeparators(int i, color Color, int linestyle, int thickness)
   {
   a = linestyle;
   b = thickness;
   c =1; if (a==0)c=b;   
   vline  = "[Chart Panel] Prior Session  Line " + (i-1);       
                                 
   if (ObjectFind(vline) != 0){ 
      if(!Separators_Thru_SubWindows) {ObjectCreate(vline, OBJ_TREND, 0, Time[i], 0, Time[i], 100);} 
      else {ObjectCreate(vline, OBJ_VLINE, 0, Time[i], 0);}     
      ObjectSet(vline, OBJPROP_STYLE, a);    
      ObjectSet(vline, OBJPROP_WIDTH, c);
      ObjectSet(vline, OBJPROP_COLOR, Color);
      ObjectSet(vline, OBJPROP_BACK, true); }
   else{
      ObjectMove(vline, 0, Time[i], 0);} 
   }  

//+-------------------------------------------------------------------------------------------+
//| TzShift sub-routine computes index of first/last bars of yesterday & first bar  of today  |
//+-------------------------------------------------------------------------------------------+

//+-------------------------------------------------------------------------------------------+
//| PivotDay sub-routine computes index of first/last bars of yesterday & first bar  of today |
//+-------------------------------------------------------------------------------------------+
void PivotDay(int &idxfirstbartoday, int &idxfirstbaryesterday,int &idxlastbaryesterday)              
   {   
   barsperday           = (1440/PERIOD_H1);         
   dayofweektoday       = TimeDayOfWeek(iTime(NULL, PERIOD_H1, 0));
   dayofweektofind      = -1;
                 
   //Due to gaps in the data, and shift of time around weekends (due to time zone)
   //it is not as easy as to just look back for a bar with 00:00 time  
   idxfirstbartoday= 0; idxfirstbaryesterday= 0;  idxlastbaryesterday= 0;     
   switch (dayofweektoday) { 
      case 0: dayofweektofind = 5; break; // Sun (Sunday considered as short trading day)
      default: dayofweektofind = dayofweektoday -1; break;}                    

   //Search backwards for the last occurence of a "today's first bar"
   for (i=1; i<=barsperday + 1; i++){
      datetime timet = iTime(NULL, PERIOD_H1, i);
      if(TimeDayOfWeek(timet) != dayofweektoday) {idxfirstbartoday = i-1;break;}}
   for (j= 0; j<=2*barsperday + 1; j++) {
      datetime timey = iTime(NULL, PERIOD_H1, i+j);
      if(TimeDayOfWeek(timey) == dayofweektofind) {idxlastbaryesterday = i+j; break;}}
   for (j= 1; j<=barsperday; j++) {
      datetime timey2 = iTime(NULL, PERIOD_H1, idxlastbaryesterday+j);
      if(TimeDayOfWeek(timey2) != dayofweektofind) {idxfirstbaryesterday = idxlastbaryesterday+j-1; break;}}
      
   //Search for Friday's data range if yesterday is a Sunday  
   if (dayofweektofind == 0)
      {
      //Search backwards for the last occurence of a "today's first bar"
      for (i=1; i<=barsperday + 1; i++){
         timet = iTime(NULL, PERIOD_H1, i);
         if(TimeDayOfWeek(timet) != dayofweektoday) {idxfirstbartoday = i-1;break;}}
      for (j= 0; j<=2*barsperday + 1; j++) {
         timey = iTime(NULL, PERIOD_H1, i+j);
         if(TimeDayOfWeek(timey) == 5) {idxlastbaryesterday = i+j; break;}}
      for (j= 1; j<=barsperday; j++) {
         timey2 = iTime(NULL, PERIOD_H1, idxlastbaryesterday+j);
         if(TimeDayOfWeek(timey2) != 5){idxfirstbaryesterday = idxlastbaryesterday+j-1;break;}}
      } 
                      
// Start - Added by CaveMan to cater for missing/partial previous day's data (both time shifted & broker/server pivots) 
   //This section takes care of the Christmas and New Year Day problem where there is a day of partial data on the eve
   //of these holidays followed by an entire day without any data at all. The workaround to handle this on market
   //open after the holiday is to use data from the day before the eve of the holiday right up to the end of the eve of
   //the holiday for pivot computation. E.g. On the 26 December market opens after Christmas. Use full day data of 23rd 
   //plus partial data of 24th Dec to compute the pivots. Same applies to New Year Day.
   int YesterdayBarsTotal = idxfirstbaryesterday - idxlastbaryesterday;
   if (YesterdayBarsTotal == 0)   
      {
      if (dayofweektofind == 0) {dayofweektofind = 4;} 
      else if (dayofweektofind == 1) {dayofweektofind = 5;} 
      else {dayofweektofind = dayofweektofind - 1;}

      //Repeat this search with the new dayofweektofind information
      for (i=1; i<=barsperday + 1; i++) {
         timet = iTime(NULL, PERIOD_H1, i);
         if(TimeDayOfWeek(timet) != dayofweektoday) {idxfirstbartoday = i-1;break;}}
      for (j= 0; j<=2*barsperday + 1; j++) {
         timey = iTime(NULL, PERIOD_H1, i+j);
         if(TimeDayOfWeek(timey) == dayofweektofind) {idxlastbaryesterday = i+j; break;}}
      for (j= 1; j<=3*barsperday; j++) {
         timey2 = iTime(NULL, PERIOD_H1, idxlastbaryesterday+j);
         if(TimeDayOfWeek(timey2) != dayofweektofind) {idxfirstbaryesterday = idxlastbaryesterday+j-1;break;}}
      YesterdayBarsTotal = idxfirstbaryesterday - idxlastbaryesterday;
      }            
   //Standard codes to handle days of partial data (including Fridays and Sundays)
   //If one hour bars < 24, use additional data from one more day back 
   if (YesterdayBarsTotal < 23)
      {
      //for Sunday, search Thursday and found Thursday partial data, then search Wednesday. 
      if (dayofweektoday == 0 && dayofweektofind == 4) {dayofweektofind = 3;} 
      //for Monday, Friday no data then include Thursday and Sunday data.
      else if (dayofweektoday == 1 && dayofweektofind == 4) {dayofweektofind = 4;} 
      else if (dayofweektofind == 0 || dayofweektofind == 1) { dayofweektofind = 5;} 
      else {dayofweektofind = dayofweektofind - 1;}
      //Search backwards for the last occurence of a "today's first bar"
      for (i=1; i<=barsperday + 1; i++) {
         timet = iTime(NULL, PERIOD_H1, i);
         if(TimeDayOfWeek(timet) != dayofweektoday) {idxfirstbartoday = i-1;break;}}
      for (j= 0; j<=3*barsperday + 1; j++) { // Need to increase another 24 hours to search
         timey = iTime(NULL, PERIOD_H1, i+j);
         if(TimeDayOfWeek(timey) == dayofweektofind) {idxlastbaryesterday = i+j; break;}}
      for (j= 1; j<=4*barsperday; j++) {
         timey2 = iTime(NULL, PERIOD_H1, idxlastbaryesterday+j);
         if(TimeDayOfWeek(timey2) != dayofweektofind){idxfirstbaryesterday = idxlastbaryesterday+j-1;break;}}                             
      //This small section handles a very unique combination of events that coincide during the short opening trading hours 
      //of the Sunday following Christmas Day and New Year Day if both of these holidays happen to fall on a Thursday. 
      //This occured on the short Sunday trading hours before Monday kicks in. For those few hours, pivot used on Friday are 
      //extended into Sunday. These pivot levels are calculated using full Thursday's data. In this unique situation, 
      //Thursday was New Year Day (major public holiday)with no data at all. This will cause TzPivot to malfunction in similar 
      //situation in the future. As a workaround, for those short few hours of Sunday, if a similar situation should occur,
      //partial data of Wednesday and Friday will be used for pivot computation. 
      YesterdayBarsTotal = idxfirstbaryesterday - idxlastbaryesterday;
      if (YesterdayBarsTotal == 0)   
         {
         if (dayofweektofind == 1) {dayofweektofind = 5;}
         else {dayofweektofind = dayofweektofind - 1;}
         //Repeat this search with the new dayofweektofind information
         for (i=1; i<=barsperday + 1; i++) {
            timet = iTime(NULL, PERIOD_H1, i);
            if(TimeDayOfWeek(timet) != dayofweektoday) {idxfirstbartoday = i-1;break;}}
         for (j= 0; j<=2*barsperday + 1; j++) {
            timey = iTime(NULL, PERIOD_H1, i+j);
            if(TimeDayOfWeek(timey) == dayofweektofind) {idxlastbaryesterday = i+j; break;}}
         for (j= 1; j<=3*barsperday; j++) {
            timey2 = iTime(NULL, PERIOD_H1, idxlastbaryesterday+j);
            if(TimeDayOfWeek(timey2) != dayofweektofind){idxfirstbaryesterday = idxlastbaryesterday+j-1;break;}}
         idxlastbaryesterday = idxfirstbartoday + 1;
         }                     
      //Include partial day's data and combine with previous day's data for pivot computation. Do not include for Sundays
      //as Sunday short trading hours uses Friday's pivots that are computed from full Thursday's data.
      //This is more for handling of holidays including Christmas and New Year Day. 
      if (dayofweektoday != 0) idxlastbaryesterday = idxfirstbartoday + 1; 
      }          
// End - Added by CaveMan to cater for missing/partial previous day's data 
   }                                

//+-------------------------------------------------------------------------------------------+
//| Indicator End                                                                             |                                                        
//+-------------------------------------------------------------------------------------------+      

